#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""SSO driver files updater."""

import os
import sys
import stat
import shutil
import logging
from pathlib import Path

from bombaat.downloader import get_url
from bombaat.downloader import extract_zip
from bombaat.downloader import download_zip
from bombaat.init import get_download_host_url
from bombaat.downloader import current_platform
from bombaat.downloader import config_executable
from bombaat.downloader import clear_cache_folders

from bombaat import __bombaat_home__

logger = logging.getLogger(__name__)

DOWNLOADS_FOLDER = Path(__bombaat_home__)
BASE_URL = os.environ.get('BOMBAAT_DOWNLOAD_HOST', get_download_host_url())

DRIVERS_FILE = 'drivers.zip'
CHROME_FOLDER = 'chrome-mac'
DRIVER_FOLDER = 'driver'

download_urls = {'mac': f'{BASE_URL}/{DRIVERS_FILE}',}

config_files = {'mac': [(DOWNLOADS_FOLDER / 'driver' / 'chromedriver'),
                        (DOWNLOADS_FOLDER / 'chrome-mac' / 'Chromium.app' /
                                        'Contents' / 'MacOS' / 'Chromium')]}

def download_driver() -> None:
    """Download and extract driver files."""
    extract_zip(download_zip(get_url(download_urls)),
                                    DOWNLOADS_FOLDER,
                                    DRIVERS_FILE)
    zip_path = DOWNLOADS_FOLDER / DRIVERS_FILE
    if check_driver() and zip_path.exists():
        zip_path.unlink()

    if not check_driver():
        raise IOError('Failed to extract drivers.')

    for exec_path in config_executable(config_files):
        if 'chrome' in str(exec_path).lower():
            exec_path.chmod(exec_path.stat().st_mode | stat.S_IXOTH |
                                            stat.S_IXGRP | stat.S_IXUSR)


def check_driver() -> bool:
    """Check if driver files are placed at correct path."""
    return config_executable(config_files)[0].exists() and \
           config_executable(config_files)[1].exists()


def get_driver_file() -> str:
    """Get profile file."""
    if check_driver():
        return config_executable(config_files)
    logger.error(('Required config files are missing, you can run'
                ' `bombaat update config` to update config files.'))
    sys.exit(1)


def update_driver() -> None:
    """update drivers"""
    chrome_folder_path = DOWNLOADS_FOLDER / CHROME_FOLDER
    if chrome_folder_path.exists():
        shutil.rmtree(chrome_folder_path)

    driver_folder_path = DOWNLOADS_FOLDER / DRIVER_FOLDER
    if driver_folder_path.exists():
        shutil.rmtree(driver_folder_path)

    clear_cache_folders()
    download_driver()




import os
import sys
import logging
from pathlib import Path

import yaml

from bombaat import __bombaat_home__, __default_download_host__

logger = logging.getLogger(__name__)

DOWNLOADS_FOLDER = Path(__bombaat_home__)

download_host_file = DOWNLOADS_FOLDER / 'downloader.yaml'

def update_downloader_path() -> None:
    """Update download url."""

    if download_host_file.exists():
        os.remove(download_host_file)

    host = input('Please enter the download host url '
                                '(ex: http://example.com): ')

    with open(download_host_file, 'w') as fh:
        yaml.dump({'host': host}, fh)

def get_download_host_url() -> str:
    """Host handler."""

    if download_host_file.exists():
        with open(download_host_file) as fh:
            host = yaml.full_load(fh)
            if host is not None and 'host' in host:
                return host['host']
    return __default_download_host__



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Common utilities."""

import os
import sys
import logging
from dateutil import tz
from collections import namedtuple

import yaml
import inquirer

from blessed import Terminal
from inquirer.themes import Theme

from bombaat.errors import SSOFileReadError
from bombaat.errors import FileNotFoundError
from bombaat.profiles import get_profile_file

logger = logging.getLogger(__name__)

term = Terminal()

class MorningSun(Theme):

    term = Terminal()

    def __init__(self) -> None:
        super(MorningSun, self).__init__()
        self.Question.mark_color = self.term.bright_white
        self.Question.brackets_color = self.term.bright_white
        self.Question.default_color = self.term.bright_white
        self.Editor.opening_prompt_color = self.term.bright_black
        self.Checkbox.selection_color = self.term.red
        self.Checkbox.selection_icon = '>'
        self.Checkbox.selected_icon = 'X'
        self.Checkbox.selected_color = self.term.yellow + self.term.bold
        self.Checkbox.unselected_color = self.term.normal
        self.Checkbox.unselected_icon = 'o'
        self.List.selection_color = self.term.black_on_lightskyblue4
        self.List.selection_cursor = '>'
        self.List.unselected_color = self.term.lightskyblue3


def read_yaml(file_name: str) -> dict:
    """Read configuration from YAML file."""

    if not os.path.exists(file_name):
        logger.error(f'Cloud config file {file_name} does not exist!')
        raise FileNotFoundError()
    try:
        with open(file_name, 'r') as fh:
            return yaml.safe_load(fh)
    except Exception as err:
        raise SSOFileReadError(
                        'Unable to read SSO config: %s' % (str(err),))
        sys.exit(1)

def write_yaml(file_name: str, data: dict) -> None:
    """YAML Writer."""
    with open(file_name, 'w') as fh:
        yaml.dump(data, fh)

def get_cloud_profiles() -> dict:
    """Get cloud profiles from yaml."""
    return read_yaml(str(get_profile_file()))


def ask(msg: str, choice_list: list) -> str:
    sys.stdout.write(term.homes + term.clear)
    _questions = [inquirer.List('choice',
                        message=term.bright_white +
                        msg +
                        term.deepskyblue,
                        choices=choice_list,),]
    _answers = inquirer.prompt(_questions,theme=MorningSun())
    return _answers


def datetime_utc_to_local(utc_time: str) -> str:
    """Convert UTC time to Local time"""
    from_zone = tz.tzutc()
    to_zone = tz.tzlocal()
    utc = utc_time.replace(tzinfo=from_zone)
    local_time = utc.astimezone(to_zone)
    local_time = local_time.strftime('%Y-%m-%d %I:%M %p')
    return local_time


def print_msg(msg):
    return sys.stdout.write(term.white(f'{msg}'))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""IDP SAML Response Handler."""

import os
import sys
import time
import base64
import logging

from urllib.parse import unquote
from urllib.parse import parse_qs
from xml.etree import ElementTree as ET
from typing import List, Tuple, Generator

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

from bombaat.utils import print_msg
from bombaat.driver import check_driver
from bombaat.errors import SAMLParserError
from bombaat.driver import get_driver_file
from bombaat.errors import SSOConnectionError

logger = logging.getLogger(__name__)

class SAMLParser(object):
    """Class to initiate SAML login with Azure AD Services."""

    def __init__(self, login_url: str, username: str, password: str) -> None:
        """Initialize SAML settings.

        :param str login_url: Should be taken from Azure Enterprise Applications
                              section. Also available in MS myapps portal.
        :param str username:  SSO username for an user.
        :param str password:  SSO password for an user.
        :return: None
        """
        self.login_url = login_url
        self.username = username
        self.password = password

        for exec_path in get_driver_file():
            if 'chromedriver' in str(exec_path).lower():
                self.executable_path = str(exec_path)
            elif 'chromium' in str(exec_path).lower():
                self.binary_location = str(exec_path)

    def run(self) -> tuple:
        """Main function."""
        saml = self._create_driver(self.login_url)
        if saml:
            arns = self._get_arn(saml['saml_response'])
            final = {**arns, **saml}
            return final

    def _create_driver(self, login_url: str,
                                        headless: bool = True) -> Generator:
        """Create and configure a chrome webdriver object."""

        options = Options()
        options.add_argument('--log-level=ALL')
        options.binary_location = self.binary_location
        options.add_experimental_option('w3c', False)

        if headless:
            options.add_argument('--headless')

        caps = DesiredCapabilities.CHROME.copy()
        caps['loggingPrefs'] = {'performance': 'ALL'}

        print_msg('connecting to SSO service...\r')
        try:
            driver = webdriver.Chrome(executable_path=self.executable_path,
                                    options=options, desired_capabilities=caps)
            driver.get(self.login_url)
            time.sleep(2)
            print_msg('processing the authentication request...\r')
            self._process_login(driver,'idSIButton9','i0116',self.username)
            username_error = self._verify_credentials(driver, 'usernameError')
            time.sleep(2)
            if not username_error:
                self._process_login(driver,'idSIButton9','i0118', self.password)
            else:
                logger.error('Incorrect username! You can update username '
                                'by running the command `bombaat configure`')
                sys.exit(1)
            password_error = self._verify_credentials(driver, 'passwordError')
            time.sleep(1)
            if not password_error:
                #self._process_login(driver, button_id='idSIButton9')
                saml = self._get_saml(driver)
                driver.quit()
                return saml
            else:
                logger.error('The password you entered is incorrect! '
                                                        'Please try again.')
                sys.exit(1)
        except Exception as err:
            logger.error('Failed to process SSO authentication, '
                                                    'Please try again later.')
            driver.quit()
            sys.exit(1)

    def _process_login(self, driver: Generator, button_id: str,
                        element_id: str = None, key_name: str = None) -> None:
        """Process user credentials."""
        try:
            if element_id:
                element = WebDriverWait(driver, 10).until(
                            EC.presence_of_element_located((By.ID, element_id)))
                element.send_keys(key_name)
                button = driver.find_element_by_id(button_id).click()
                time.sleep(1)
            else:
                button = driver.find_element_by_id(button_id).click()
        except TimeoutException as err:
            logger.error('Your request has been timed out! '
                                            'please try again later!')
            sys.exit(1)

    @staticmethod
    def _verify_credentials(driver: Generator, id: str) -> None:
        """Verify user credentials."""
        print_msg('verifying Single Sign On login credentials...\r')
        try:
            if driver.find_element_by_id(id):
                driver.quit()
                return True
        except:
            pass

    def _get_saml(self, driver: Generator, sleeptime: float = 0.5) -> str:
        """Get SAML Response from WebDriver Obj."""

        saml_url = 'https://signin.aws.amazon.com/saml'

        print_msg('waiting for SAML response from single sign on...\r')
        while True:
            saml_response = {}
            time.sleep(sleeptime)
            for line in driver.get_log('performance'):
                try:
                    if 'samlresponse' in str(line).lower() and \
                                                saml_url in str(line).lower():
                        saml = [k[0].split(',')[0][:-1] \
                                  for k in parse_qs(line['message']).values()]
                        saml_response['saml_response'] = unquote(saml[0])
                        return saml_response
                except Exception as err:
                    raise SAMLParserError(
                     'Unable to parse SAML Response from SSO: %s' % (str(err),))

    def _get_arn(self, saml: str) -> List:
        """Get AWS Roles and Principal ARN from SAML Response."""

        attribute = '{urn:oasis:names:tc:SAML:2.0:assertion}Attribute'
        attribute_name = 'https://aws.amazon.com/SAML/Attributes/Role'
        attribute_val = '{urn:oasis:names:tc:SAML:2.0:assertion}AttributeValue'

        arns = {}
        for attrib in ET.fromstring(base64.b64decode(saml)).iter(attribute):
            if (attrib.get('Name') == attribute_name):
                for value in attrib.iter(attribute_val):
                     if arns.get('roles_arn') is None:
                         arns['roles_arn'] = [value.text.split(',')[0]]
                     else:
                         arns['roles_arn'].append(value.text.split(',')[0])

                     # Principal ARN for Identity Provider in AWS is same for
                     # all roles, all attribute value will have role arn and
                     # principal arn, so keeping only one.
                     if arns.get('principal_arn') is None:
                         arns['principal_arn'] = [value.text.split(',')[1]]
                     else:
                         val = value.text.split(',')[1]
                         if val not in arns['principal_arn']:
                             arns['principal_arn'].append(val)
        return arns



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Cloud Profile Management"""

import os
import logging
from distutils.util import strtobool

from blessed import Terminal
from botocore.configloader import load_config
from botocore.exceptions import ProfileNotFound
from botocore.session import Session, get_session
from awscli.customizations.configure.writer import ConfigFileWriter

from bombaat.utils import ask
from bombaat.utils import read_yaml
from bombaat.utils import MorningSun
from bombaat.driver import check_driver
from bombaat.driver import download_driver
from bombaat.utils import get_cloud_profiles

logger = logging.getLogger(__name__)

class Configure(object):
    """Configure cloud profiles."""

    # If you want to add new values to prompt, update this list here.
    VALUES_TO_PROMPT = [
      # (logical_name, config_name, prompt_text)
      ('username', 'Enter Azure SSO username (email)'),
      ('default', "Do you want to make it the 'default' profile? (True/False)"),
    ]

    def __init__(self) -> None:
        super(Configure, self).__init__()

        if not check_driver():
            logger.warning('\nConfiguration files missing, please run '
                '`bombaat update all` to install latest configuration files.')

        self._cloud_profile = self._get_cloud_account_name()
        self._session = Session(profile=self._cloud_profile)
        self._config_writer = ConfigFileWriter()

    def main(self) -> None:
        """Main function."""
        new_values = {}
        try:
            config = self._session.get_scoped_config()
        except ProfileNotFound:
            config = {}
        for config_name, prompt_text in self.VALUES_TO_PROMPT:
            current_value = config.get(config_name)
            new_value = self._get_value(current_value, prompt_text)
            if new_value is not None and new_value != current_value and \
                                                      config_name == 'default':
                if new_value.lower() == 'true':
                    available_profiles = self._session.available_profiles
                    for profile in available_profiles:
                        self._session = Session(profile=profile)
                        profile_config = self._session.get_scoped_config()
                        default_val = profile_config.get('default')
                        if default_val is not None and \
                                                bool(strtobool(default_val)):
                            update_default_values = {}
                            update_default_values['default'] = False
                            self._update_config_file(
                                                update_default_values, profile)
                    new_values[config_name] = bool(strtobool(new_value))
                else:
                    new_values[config_name] = False
            if new_value is not None and new_value != current_value and \
                                                    config_name == 'username':
                new_values[config_name] = new_value

        self._update_config_file(new_values, self._cloud_profile)
        configure_default_profile()

    def _get_cloud_account_name(self) -> str:
        """Get cloud profile from user.

        :param str file_name: yaml config file which has cloud profiles.
        :return: Cloud account name chosen by user.

        Yaml file should be in the following format:
        Example:
                login_urls:
                  devcloud:
                    name: 'Friendly Cloud Name'
                    login_url: 'login url collected from Azure AD'
                  qacloud:
                    name: 'Friendly Cloud Name'
                    login_url: 'login url collected from Azure AD'
                  prodcloud:
                    name: 'Friendly Cloud Name'
                    login_url: 'login url collected from Azure AD'

        You can add 'n' number of account information.
        """
        account_names = get_cloud_profiles()
        friendly_names = list()
        for account in account_names['login_urls']:
            friendly_names.append(account_names['login_urls'][account]['name'])
        msg = 'Choose the cloud account you would like to configure'
        _answers = ask(msg, friendly_names)
        if _answers:
            choice = _answers.get('choice').strip()
            for key, val in account_names['login_urls'].items():
                if val['name'] == choice:
                    return key

    def _get_value(self, current_value, prompt_text='') -> str:
        """Manage user inputs.

        :param str current_value: value currently configured
        :param str prompt_text: a string
        :return:
        """
        response = input(f'{prompt_text} [{current_value}]: ')
        if not response:
            # If the user hits enter, we return a value of None
            # instead of an empty string.  That way we can determine
            # whether or not a value has changed.
            response = None
        return response

    def _update_config_file(self, new_values, profile) -> int:
        """Setup config file.

        :param dict new_values: dictionary containing config values
        :param str profile: Name of the cloud profile
        :return:
        """
        config_filename = os.path.expanduser(
            self._session.get_config_variable('config_file'))
        if new_values:
            new_values['__section__'] = ('profile {}'.format(profile))
            self._config_writer.update_config(new_values, config_filename)
        return 0


def write_credentials(session, new_values, profile=None) -> None:
    """Credentials Writer."""

    credential_values = {}
    if 'AccessKeyId' in new_values:
        credential_values['aws_access_key_id'] = new_values.pop('AccessKeyId')
    if 'SecretAccessKey' in new_values:
        credential_values['aws_secret_access_key'] = new_values.pop(
                                                            'SecretAccessKey')
    if 'SessionToken' in new_values:
        credential_values['aws_security_token'] = new_values.pop('SessionToken')

    if credential_values:
        if profile is not None:
            credential_values['__section__'] = profile
        credentials_filename = os.path.expanduser(
            session.get_config_variable('credentials_file'))
        config_writer = ConfigFileWriter()
        config_writer.update_config(credential_values,
                                          credentials_filename)


def configure_default_profile() -> None:
    """Configure default profile."""

    session = get_session()
    config_file = session.get_config_variable('config_file')
    config = load_config(config_file)
    cloud_profiles = get_cloud_profiles()

    available_profiles = list(config['profiles'].keys())
    account_names = list(cloud_profiles['login_urls'].keys())
    login_profiles = list(set(available_profiles).intersection(account_names))

    new_values = dict()
    for profile in login_profiles:
        if config['profiles'][profile].get('default') is not None:
            if bool(strtobool(config['profiles'][profile]['default'])):
                profile_values = session.full_config['profiles'][profile]
                if 'aws_access_key_id' in profile_values:
                    new_values['AccessKeyId'] = profile_values.pop(
                                                          'aws_access_key_id')
                if 'aws_secret_access_key' in profile_values:
                    new_values['SecretAccessKey'] = profile_values.pop(
                                                      'aws_secret_access_key')
                if 'aws_security_token' in profile_values:
                    new_values['SessionToken'] = profile_values.pop(
                                                         'aws_security_token')
                write_credentials(session, new_values)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""SSO config files updater module."""

import os
import sys
import shutil
import logging
import subprocess
from io import BytesIO
from pathlib import Path
from zipfile import ZipFile

import urllib3
from tqdm import tqdm

from bombaat import __bombaat_home__

logger = logging.getLogger(__name__)

DOWNLOADS_FOLDER = Path(__bombaat_home__)

NO_PROGRESS_BAR = os.environ.get('BOMBAAT_NO_PROGRESS_BAR', '')

if NO_PROGRESS_BAR.lower() in ('1', 'true'):
    NO_PROGRESS_BAR = True  # type: ignore


def current_platform() -> str:
    """Get current platform name by short string."""

    if sys.platform.startswith('darwin'):
        return 'mac'
    raise OSError('Unsupported platform: ' + sys.platform)


def get_url(download_urls: dict) -> str:
    """Get download url."""
    return download_urls[current_platform()]


def download_zip(url: str) -> BytesIO:
    """Download data from url."""

    # disable warnings so that we don't need a cert.
    # see https://urllib3.readthedocs.io/en/latest/advanced-usage.html for more
    urllib3.disable_warnings()

    with urllib3.PoolManager() as http:
        # Get data from url.
        # set preload_content=False means using stream later.
        data = http.request('GET', url, preload_content=False)

        try:
            total_length = int(data.headers['content-length'])
        except (KeyError, ValueError, AttributeError):
            total_length = 0

        process_bar = tqdm(total=total_length,
                                file=os.devnull if NO_PROGRESS_BAR else None,)

        # 10 * 1024
        _data = BytesIO()
        for chunk in data.stream(10240):
            _data.write(chunk)
            process_bar.update(len(chunk))
        process_bar.close()

    return _data


def extract_zip(data: BytesIO, path: Path, download_file: str) -> None:
    """Extract zipped data to path."""

    msg = (f'Failed to update SSO configuration, something went wrong.')

    # On mac zipfile module cannot extract correctly, so use unzip instead.
    if current_platform() == 'mac':
        zip_path = path / download_file
        if not path.exists():
            path.mkdir(parents=True)
        with zip_path.open('wb') as f:
            f.write(data.getvalue())
        if not shutil.which('unzip'):
            logger.error(msg)
            sys.exit(1)

        proc = subprocess.run(['unzip', str(zip_path)],
                                cwd=str(path),
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT,)

        if proc.returncode != 0:
            sys.stdout.write(proc.stdout.decode())
            logger.error(msg)
            sys.exit(1)
    else:
        with ZipFile(data) as zf:
            zf.extractall(str(path))


def config_executable(config_files: dict) -> Path:
    """Get path of the executable."""
    return config_files[current_platform()]


def clear_cache_folders() -> None:
    """Clear cache store to avoid zip extraction."""
    CACHE_FOLDER = '__MACOSX'
    cache_folder_path = DOWNLOADS_FOLDER / CACHE_FOLDER
    if cache_folder_path.exists():
        shutil.rmtree(cache_folder_path)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""SSO profile files udpater."""

import os
import sys
import stat
import shutil
import logging
from pathlib import Path

from bombaat.downloader import get_url
from bombaat.downloader import extract_zip
from bombaat.downloader import download_zip
from bombaat.init import get_download_host_url
from bombaat.downloader import current_platform
from bombaat.downloader import config_executable
from bombaat.downloader import clear_cache_folders

from bombaat import __bombaat_home__

logger = logging.getLogger(__name__)

DOWNLOADS_FOLDER = Path(__bombaat_home__)
BASE_URL = os.environ.get('BOMBAAT_DOWNLOAD_HOST', get_download_host_url())

PROFILE_FILE = 'profiles.zip'
PROFILE_FOLDER = 'profiles'

download_urls = {'mac': f'{BASE_URL}/{PROFILE_FILE}',}
config_files = {'mac': [(DOWNLOADS_FOLDER / PROFILE_FOLDER / 'sso.yml')]}


def download_profile() -> None:
    """Download and extract profile file."""

    profile_folder_path = DOWNLOADS_FOLDER / PROFILE_FOLDER
    if profile_folder_path.exists():
        shutil.rmtree(profile_folder_path)

    clear_cache_folders()

    extract_zip(download_zip(get_url(download_urls)),
                                    DOWNLOADS_FOLDER,
                                    PROFILE_FILE)

    zip_path = DOWNLOADS_FOLDER / PROFILE_FILE

    if zip_path.exists():
        zip_path.unlink()

    if not check_profile():
        logger.error('Failed to update cloud profiles.')


def check_profile() -> bool:
    """Check if profile files are placed at correct path."""
    return config_executable(config_files)[0].exists()


def get_profile_file() -> str:
    """Get profile file."""
    if check_profile():
        return config_executable(config_files)[0]
    logger.error('\nLooks like SSO profile file is missing, '
                    'you can run `bombaat update profile` to udpate profiles.')
    sys.exit(1)
